/*
 * MessagePack for C utilities
 *
 * Copyright (C) 2014 FURUHASHI Sadayuki
 *
 *    Distributed under the Boost Software License, Version 1.0.
 *    (See accompanying file LICENSE_1_0.txt or copy at
 *    http://www.boost.org/LICENSE_1_0.txt)
 */
#ifndef MSGPACK_UTIL_H
#define MSGPACK_UTIL_H

#define MSGPACK_UNUSED(a) (void)(a)

#endif /* MSGPACK_UTIL_H */
