/*
Copyright Rene Rivera 2015-2016
Distributed under the Boost Software License, Version 1.0.
(See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)
*/

#ifndef MSGPACK_PREDEF_VERSION_H
#define MSGPACK_PREDEF_VERSION_H

#include <msgpack/predef/version_number.h>

#define MSGPACK_PREDEF_VERSION MSGPACK_VERSION_NUMBER(1,4,1)

#endif
